@@ -1,10 +1,7 @@
-
 """Useful functions and classes for database scripts.
-
 """
 
-from __future__ import division, absolute_import, print_function
-
+import argparse
 import errno
 import logging
 import logging.config
@@ -25,13 +22,16 @@ try:
 except ImportError:
     default_skylog = 0
 
-__pychecker__ = 'no-badexcept'
 
-__all__ = ['BaseScript', 'UsageError', 'daemonize', 'DBScript']
+__all__ = (
+    'BaseScript', 'UsageError', 'daemonize', 'DBScript',
+)
+
 
 class UsageError(Exception):
     """User induced error."""
 
+
 #
 # daemon mode
 #
@@ -58,6 +58,7 @@ def daemonize():
     if fd > 2:
         os.close(fd)
 
+
 #
 # Pidfile locking+cleanup & daemonization combined
 #
@@ -91,7 +92,9 @@ def run_single_process(runnable, daemon, pidfile):
         if own_pidfile:
             try:
                 os.remove(pidfile)
-            except: pass
+            except BaseException:
+                pass
+
 
 #
 # logging setup
@@ -100,20 +103,6 @@ def run_single_process(runnable, daemon, pidfile):
 _log_config_done = 0
 _log_init_done = {}
 
-def _load_log_config(fn, defs):
-    """Fixed fileConfig."""
-
-    # Work around fileConfig default behaviour to disable
-    # not only old handlers on load (which slightly makes sense)
-    # but also old logger objects (which does not make sense).
-
-    if sys.hexversion >= 0x2060000:
-        logging.config.fileConfig(fn, defs, False)
-    else:
-        logging.config.fileConfig(fn, defs)
-        root = logging.getLogger()
-        for lg in root.manager.loggerDict.values():
-            lg.disabled = 0
 
 def _init_log(job_name, service_name, cf, log_level, is_daemon):
     """Logging setup happens here."""
@@ -144,7 +133,7 @@ def _init_log(job_name, service_name, cf, log_level, is_daemon):
             fn = os.path.expanduser(fn)
             if os.path.isfile(fn):
                 defs = {'job_name': job_name, 'service_name': service_name}
-                _load_log_config(fn, defs)
+                logging.config.fileConfig(fn, defs, False)
                 got_skylog = 1
                 break
         _log_config_done = 1
@@ -164,7 +153,7 @@ def _init_log(job_name, service_name, cf, log_level, is_daemon):
 
     # compatibility: specify ini file in script config
     def_fmt = '%(asctime)s %(process)s %(levelname)s %(message)s'
-    def_datefmt = '' # None
+    def_datefmt = ''  # None
     logfile = cf.getfile("logfile", "")
     if logfile:
         fstr = cf.get('logfmt_file', def_fmt)
@@ -173,10 +162,10 @@ def _init_log(job_name, service_name, cf, log_level, is_daemon):
             fstr = cf.get('logfmt_file_verbose', fstr)
             fstr_date = cf.get('logdatefmt_file_verbose', fstr_date)
         fmt = logging.Formatter(fstr, fstr_date)
-        size = cf.getint('log_size', 10*1024*1024)
+        size = cf.getint('log_size', 10 * 1024 * 1024)
         num = cf.getint('log_count', 3)
         file_hdlr = logging.handlers.RotatingFileHandler(
-                    logfile, 'a', size, num)
+            logfile, 'a', size, num)
         file_hdlr.setFormatter(fmt)
         root.addHandler(file_hdlr)
 
@@ -195,7 +184,7 @@ def _init_log(job_name, service_name, cf, log_level, is_daemon):
     return log
 
 
-class BaseScript(object):
+class BaseScript:
     """Base class for service scripts.
 
     Handles logging, daemonizing, config, errors.
@@ -257,6 +246,9 @@ class BaseScript(object):
     # start time
     started = 0
 
+    # set to True to use argparse
+    ARGPARSE = False
+
     def __init__(self, service_name, args):
         """Script setup.
 
@@ -278,8 +270,7 @@ class BaseScript(object):
         self.log_level = logging.INFO
 
         # parse command line
-        parser = self.init_optparse()
-        self.options, self.args = parser.parse_args(args)
+        self.options, self.args = self.parse_args(args)
 
         # check args
         if self.options.version:
@@ -319,6 +310,16 @@ class BaseScript(object):
         elif self.options.cmd == "reload":
             self.send_signal(signal.SIGHUP)
 
+    def parse_args(self, args):
+        if self.ARGPARSE:
+            parser = self.init_argparse()
+            options = parser.parse_args(args)
+            args = getattr(options, "args", [])
+        else:
+            parser = self.init_optparse()
+            options, args = parser.parse_args(args)
+        return options, args
+
     def print_version(self):
         service = self.service_name
         ver = getattr(self, '__version__', None)
@@ -353,7 +354,7 @@ class BaseScript(object):
         pos = doc and doc.rfind('::\n') or -1
         if pos < 0:
             return
-        doc = doc[pos+2 : ].rstrip()
+        doc = doc[pos + 2:].rstrip()
         doc = skytools.dedent(doc)
 
         # merge overrided options into output
@@ -422,9 +423,9 @@ class BaseScript(object):
         p.add_option("-V", "--version", action="store_true",
                      help="print version info and exit")
         p.add_option("", "--ini", action="store_true",
-                    help="display sample ini file")
+                     help="display sample ini file")
         p.add_option("", "--set", action="append",
-                    help="override config setting (--set 'PARAM=VAL')")
+                     help="override config setting (--set 'PARAM=VAL')")
 
         # control options
         g = optparse.OptionGroup(p, 'control running process')
@@ -441,6 +442,52 @@ class BaseScript(object):
 
         return p
 
+    def init_argparse(self, parser=None):
+        """Initialize a ArgumentParser() instance that will be used to
+        parse command line arguments.
+
+        Note that it can be overridden both directions - either BaseScript
+        will initialize an instance and pass it to user code or user can
+        initialize and then pass to BaseScript.init_optparse().
+
+        @param parser: optional ArgumentParser() instance,
+               where BaseScript should attach its own arguments.
+        @return: initialized ArgumentParser() instance.
+        """
+        if parser:
+            p = parser
+        else:
+            p = argparse.ArgumentParser()
+
+        # generic options
+        p.add_argument("-q", "--quiet", action="store_true",
+                       help="log only errors and warnings")
+        p.add_argument("-v", "--verbose", action="count",
+                       help="log verbosely")
+        p.add_argument("-d", "--daemon", action="store_true",
+                       help="go background")
+        p.add_argument("-V", "--version", action="store_true",
+                       help="print version info and exit")
+        p.add_argument("--ini", action="store_true",
+                       help="display sample ini file")
+        p.add_argument("--set", action="append",
+                       help="override config setting (--set 'PARAM=VAL')")
+        p.add_argument("args", nargs="*")
+
+        # control options
+        g = p.add_argument_group('control running process')
+        g.add_argument("-r", "--reload",
+                       action="store_const", const="reload", dest="cmd",
+                       help="reload config (send SIGHUP)")
+        g.add_argument("-s", "--stop",
+                       action="store_const", const="stop", dest="cmd",
+                       help="stop program safely (send SIGINT)")
+        g.add_argument("-k", "--kill",
+                       action="store_const", const="kill", dest="cmd",
+                       help="kill program immediately (send SIGTERM)")
+
+        return p
+
     def send_signal(self, sig):
         if not self.pidfile:
             self.log.warning("No pidfile in config, nothing to do")
@@ -487,8 +534,8 @@ class BaseScript(object):
         self.loop_delay = self.cf.getfloat("loop_delay", self.loop_delay)
         self.exception_sleep = self.cf.getfloat("exception_sleep", 20)
         self.exception_quiet = self.cf.getlist("exception_quiet", [])
-        self.exception_grace = self.cf.getfloat("exception_grace", 5*60)
-        self.exception_reset = self.cf.getfloat("exception_reset", 15*60)
+        self.exception_grace = self.cf.getfloat("exception_grace", 5 * 60)
+        self.exception_reset = self.cf.getfloat("exception_reset", 15 * 60)
 
     def hook_sighup(self, sig, frame):
         "Internal SIGHUP handler.  Minimal code here."
@@ -547,7 +594,7 @@ class BaseScript(object):
         # run startup, safely
         self.run_func_safely(self.startup)
 
-        while 1:
+        while True:
             # reload config, if needed
             if self.need_reload:
                 self.reload()
@@ -594,8 +641,8 @@ class BaseScript(object):
         except UsageError as d:
             self.log.error(str(d))
             sys.exit(1)
-        except MemoryError as d:
-            try: # complex logging may not succeed
+        except MemoryError:
+            try:  # complex logging may not succeed
                 self.log.exception("Job %s out of memory, exiting", self.job_name)
             except MemoryError:
                 self.log.fatal("Out of memory")
@@ -606,16 +653,16 @@ class BaseScript(object):
                 self.log.info("got SystemExit(%s), exiting", str(d))
             self.reset()
             raise d
-        except KeyboardInterrupt as d:
+        except KeyboardInterrupt:
             self.send_stats()
             if prefer_looping and self.looping and self.loop_delay > 0:
                 self.log.info("got KeyboardInterrupt, exiting")
             self.reset()
             sys.exit(1)
         except Exception as d:
-            try: # this may fail too
+            try:  # this may fail too
                 self.send_stats()
-            except:
+            except BaseException:
                 pass
             if self.last_func_fail is None:
                 self.last_func_fail = time.time()
@@ -700,12 +747,14 @@ class BaseScript(object):
     stat_add = stat_put                 # Old, deprecated function.
     stat_inc = stat_increase
 
+
 ##
 ##  DBScript
 ##
 
 #: how old connections need to be closed
-DEF_CONN_AGE = 20*60  # 20 min
+DEF_CONN_AGE = 20 * 60  # 20 min
+
 
 class DBScript(BaseScript):
     """Base class for database scripts.
@@ -735,7 +784,7 @@ class DBScript(BaseScript):
         """
         self.db_cache = {}
         self._db_defaults = {}
-        self._listen_map = {} # dbname: channel_list
+        self._listen_map = {}  # dbname: channel_list
         super(DBScript, self).__init__(service_name, args)
 
     def connection_hook(self, dbname, conn):
@@ -846,7 +895,7 @@ class DBScript(BaseScript):
             sql = getattr(curs, 'query', None) or '?'
             if isinstance(sql, bytes):
                 sql = sql.decode('utf8')
-            if len(sql) > 200: # avoid logging londiste huge batched queries
+            if len(sql) > 200:  # avoid logging londiste huge batched queries
                 sql = sql[:60] + " ..."
             lm = "Job %s got error on connection '%s': %s.   Query: %s" % (
                 self.job_name, cname, emsg, sql)
@@ -1004,7 +1053,7 @@ class DBScript(BaseScript):
                 if not sql_retry or tried >= sql_retry_max_count or time.time() - stime >= sql_retry_max_time:
                     raise
                 self.log.info("Job %s got error on connection %s: %s", self.job_name, dbname, e)
-            except:
+            except BaseException:
                 raise
             # y = a + bx , apply cap
             y = sql_retry_formula_a + sql_retry_formula_b * tried
@@ -1046,7 +1095,8 @@ class DBScript(BaseScript):
         except ValueError:
             pass
 
-class DBCachedConn(object):
+
+class DBCachedConn:
     """Cache a db connection."""
     def __init__(self, name, loc, max_age=DEF_CONN_AGE, verbose=False, setup_func=None, channels=()):
         self.name = name
@@ -1127,10 +1177,12 @@ class DBCachedConn(object):
         # close
         try:
             conn.close()
-        except: pass
+        except BaseException:
+            pass
 
     def check_connstr(self, connstr):
         """Drop connection if connect string has changed.
         """
         if self.loc != connstr:
             self.reset()
+
